/*
 * ChaosExplorer source file
 * Copyright (C) 2002-2003 by Roland Riegel <feedback@roland-riegel.de>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * $Id$
 *
 */

#include "chaosexplorerwindow.h"

#include "docwindow.h"
#include "mandelbrot.h"
#include "feigenbaum.h"
#include "sizewindow.h"
#include "infowindow.h"
#include "icons.h"

#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qapplication.h>
#include <qworkspace.h>
#include <qmenubar.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qpopupmenu.h>
#include <qaction.h>
#include <qprinter.h>
#include <qfileinfo.h>
#include <qfile.h>
#include <qdatastream.h>

ChaosExplorerWindow::ChaosExplorerWindow()
	: QMainWindow( 0, 0, WDestructiveClose )
{
	setCaption( "ChaosExplorer" );
	setIcon( QPixmap( app_icon_small_xpm ) );

	// create MDI workspace
	m_workspace = new QWorkspace( this );
	connect( m_workspace, SIGNAL( windowActivated( QWidget* ) ), SLOT( windowActivated( QWidget* ) ) );
	m_workspace->setBackgroundMode( PaletteDark );
	setCentralWidget( m_workspace );

	// create menus
	QPopupMenu* fileMenu = new QPopupMenu( this, "filemenu" );
	QPopupMenu* newMenu = new QPopupMenu( this );
	QPopupMenu* editMenu = new QPopupMenu( this, "editmenu" );
	QPopupMenu* sizeMenu = new QPopupMenu( this );
	QPopupMenu* windowMenu = new QPopupMenu( this, "windowmenu" );
	QPopupMenu* helpMenu = new QPopupMenu( this, "helpmenu" );

	// create toolbars
	QToolBar* fileToolBar = new QToolBar( "Standard", this );
	QToolBar* utilToolBar = new QToolBar( "Werkzeuge", this );

	// insert menus into the menu bar
	menuBar()->insertItem( "&Datei", fileMenu );
	menuBar()->insertItem( "&Bearbeiten", editMenu );
	menuBar()->insertItem( "&Fenster", windowMenu );
	menuBar()->insertItem( "&Hilfe", helpMenu );

	// add toolbars to the main window
	addToolBar( fileToolBar );
	addToolBar( utilToolBar );
	
	// create action groups for toggling
	new QActionGroup( this, "mouseutilitytoggle", true );
	new QActionGroup( this, "picsizetoggle", true );

	// create actions
	new QAction( "Schaltet auf das Werkzeug \"Bereichsauswahl\" um", QPixmap( magnifier_xpm ), "Bereichs&auswahl", 0, getActionGroup( "mouseutilitytoggle" ), "magnifier", true );
	new QAction( "Schaltet auf das Werkzeug \"Zentrieren\" um", QPixmap( center_xpm ), "&Zentrieren", 0, getActionGroup( "mouseutilitytoggle" ), "center", true );
	new QAction( "Schaltet auf das Werkzeug \"Verschieben\" um", QPixmap( move_xpm ), "Ver&schieben", 0, getActionGroup( "mouseutilitytoggle" ), "move", true );
	new QAction( "Zoomt in die Mitte des Bildes", QPixmap( zoom_in_xpm ), "&Vergrern", 0, this, "zoomin" );
	new QAction( "Zoomt aus dem Bild heraus", QPixmap( zoom_out_xpm ), "&Verkleinern", 0, this, "zoomout" );
	new QAction( "Erzeugt ein neues Mandelbrot-Dokument", QPixmap( mb_icon_xpm ), "&Mandelbrot", CTRL+Key_M, this, "newmandelbrot" );
	new QAction( "Erzeugt ein neues Feigenbaum-Dokument", QPixmap( fb_icon_xpm ), "&Feigenbaum", CTRL+Key_F, this, "newfeigenbaum" );
	new QAction( "ffnet ein bereits vorhandenes Dokument", QPixmap( open_xpm ), "&ffnen", CTRL+Key_O, this, "open" );
	new QAction( "Speichert das akt. Dokument", QPixmap( save_xpm ), "&Speichern", CTRL+Key_S, this, "save" );
	new QAction( "Speichert das akt. Dokument unter einem anderen Namen", "Speichern &unter...", 0, this, "saveas" );
	new QAction( "Exportiert das akt. Dokument in ein Bitmap", QPixmap( camera_xpm ), "Bild in &Datei exportieren...", CTRL+Key_E, this, "export" );
	new QAction( "Druckt das akt. Dokument", QPixmap( print_xpm ), "&Drucken...", CTRL+Key_P, this, "print" );
	new QAction( "Beendet ChaosExplorer", "Be&enden", CTRL+Key_Q, this, "quit" );
	new QAction( "Macht die letzte Aktion rckgngig", QPixmap( undo_xpm ), "&Rckgngig", CTRL+Key_Z, this, "undo" );
	new QAction( "Stellt die letzte Aktion wieder her", QPixmap( redo_xpm ), "&Wiederherstellen", CTRL+Key_Y, this, "redo" );
	new QAction( "", "320 x 240 Pixel", 0, getActionGroup( "picsizetoggle" ), "320", true );
	new QAction( "", "640 x 480 Pixel", 0, getActionGroup( "picsizetoggle" ), "640", true );
	new QAction( "", "1280 x 960 Pixel", 0, getActionGroup( "picsizetoggle" ), "1280", true );
	new QAction( "Benutzerdefinierte Bildgre", "&Benutzerdefiniert...", 0, getActionGroup( "picsizetoggle" ), "picsizeuser", true );
	new QAction( "Startet die Berechnung neu", QPixmap( start_xpm ), "Berechnung &neu starten", CTRL+Key_R, this, "start" );
	new QAction( "Stoppt die laufende Berechnung", QPixmap( stop_xpm ), "Berechnung sto&ppen", CTRL+Key_K, this, "stop" );
	new QAction( "Einstellungen fr das akt. Dokument bearbeiten", QPixmap( prop_xpm ), "Ein&stellungen...", CTRL+Key_D, this, "prop" );
	new QAction( "Zeigt Informationen ber ChaosExplorer an", "&Info...", CTRL+Key_I, this, "info" );

	// connect actions with their functions
	connect( windowMenu, SIGNAL( aboutToShow() ), SLOT( updateWindowMenu() ) );
	connect( getAction( "newmandelbrot" ), SIGNAL( activated() ), SLOT( fileNewMandelbrot() ) );
	connect( getAction( "newfeigenbaum" ), SIGNAL( activated() ), SLOT( fileNewFeigenbaum() ) );
	connect( getAction( "open" ), SIGNAL( activated() ), SLOT( fileOpen() ) );
	connect( getAction( "save" ), SIGNAL( activated() ), SLOT( fileSave() ) );
	connect( getAction( "saveas" ), SIGNAL( activated() ), SLOT( fileSaveAs() ) );
	connect( getAction( "export" ), SIGNAL( activated() ), SLOT( fileExport() ) );
	connect( getAction( "print" ), SIGNAL( activated() ), SLOT( filePrint() ) );
	connect( getAction( "quit" ), SIGNAL( activated() ), SLOT( fileQuit() ) );
	connect( getAction( "undo" ), SIGNAL( activated() ), SLOT( editUndo() ) );
	connect( getAction( "redo" ), SIGNAL( activated() ), SLOT( editRedo() ) );
	connect( getAction( "picsizeuser" ), SIGNAL( activated() ), SLOT( editPictureSizeUserDef() ) );
	connect( getAction( "start" ), SIGNAL( activated() ), SLOT( editStartCalc() ) );
	connect( getAction( "stop" ), SIGNAL( activated() ), SLOT( editStopCalc() ) );
	connect( getAction( "prop" ), SIGNAL( activated() ), SLOT( editProps() ) );
	connect( getAction( "info" ), SIGNAL( activated() ), SLOT( showInfo() ) );
	connect( getAction( "zoomin" ), SIGNAL( activated() ), SLOT( zoomIn() ) );
	connect( getAction( "zoomout" ), SIGNAL( activated() ), SLOT( zoomOut() ) );
	connect( getActionGroup( "picsizetoggle" ), SIGNAL( selected( QAction* ) ), SLOT( editPictureSize( QAction* ) ) );
	connect( getActionGroup( "mouseutilitytoggle" ), SIGNAL( selected( QAction* ) ), SLOT( changeUtility( QAction* ) ) );

	// insert actions into the menu
	fileMenu->insertItem( QPixmap( new_xpm ), "&Neu", newMenu );
	getAction( "open" )->addTo( fileMenu );
	getAction( "save" )->addTo( fileMenu );
	getAction( "saveas" )->addTo( fileMenu );
	fileMenu->insertSeparator();
	getAction( "export" )->addTo( fileMenu );
	getAction( "print" )->addTo( fileMenu );
	fileMenu->insertSeparator();
	getAction( "quit" )->addTo( fileMenu );

	getAction( "undo" )->addTo( editMenu );
	getAction( "redo" )->addTo( editMenu );
	editMenu->insertSeparator();
	getAction( "start" )->addTo( editMenu );
	getAction( "stop" )->addTo( editMenu );
	editMenu->insertSeparator();
	editMenu->insertItem( "Bild&gre", sizeMenu );
	editMenu->insertSeparator();
	getAction( "prop" )->addTo( editMenu );

	getAction( "newmandelbrot" )->addTo( newMenu );
	getAction( "newfeigenbaum" )->addTo( newMenu );

	getAction( "320" )->addTo( sizeMenu );
	getAction( "640" )->addTo( sizeMenu );
	getAction( "1280" )->addTo( sizeMenu );
	sizeMenu->insertSeparator();
	getAction( "picsizeuser" )->addTo( sizeMenu );

	getAction( "info" )->addTo( helpMenu );

	// add actions to the toolbars
	QToolButton* toolbutton = new QToolButton( QPixmap( new_xpm ), "", "", this, SLOT( fileNewMandelbrot() ), fileToolBar, "new" );
	toolbutton->setPopup( newMenu );
	getAction( "open" )->addTo( fileToolBar );
	getAction( "save" )->addTo( fileToolBar );
	fileToolBar->addSeparator();
	getAction( "export" )->addTo( fileToolBar );
	getAction( "print" )->addTo( fileToolBar );
	fileToolBar->addSeparator();
	getAction( "start" )->addTo( fileToolBar );
	getAction( "stop" )->addTo( fileToolBar );

	getActionGroup( "mouseutilitytoggle" )->addTo( utilToolBar );
	utilToolBar->addSeparator();
	getAction( "zoomin" )->addTo( utilToolBar );
	getAction( "zoomout" )->addTo( utilToolBar );
	utilToolBar->addSeparator();
	getAction( "prop" )->addTo( utilToolBar );
	utilToolBar->addSeparator();
	getAction( "undo" )->addTo( utilToolBar );
	getAction( "redo" )->addTo( utilToolBar );

	// open the files given on the command line
	for( int i = 1; i < qApp->argc(); i++ )
	{
		QFileInfo fi( qApp->argv()[i] );
		if( fi.exists() && fi.isFile() )
			openFile( qApp->argv()[i] );
	}

	windowActivated(0);
}

ChaosExplorerWindow::~ChaosExplorerWindow()
{
	// delete mdi windows to stop calculations
	QWidgetList list = m_workspace->windowList();
	for( QWidget* window = list.first(); window != 0; window = list.next() )
	{
		DocWindow *docwin = (DocWindow *) window;
		if( docwin->doc()->calcing() )
		{
			docwin->hide();
			delete docwin;
		}
	}
}

QAction* ChaosExplorerWindow::getAction( const char* name )
{
	return (QAction*) child( name, "QAction" );
}

QActionGroup* ChaosExplorerWindow::getActionGroup( const char* name )
{
	return (QActionGroup*) child( name, "QActionGroup" );
}

void ChaosExplorerWindow::windowActivated( QWidget* w )
{
	DocWindow *docwin = (DocWindow *) m_workspace->activeWindow();

	getActionGroup( "mouseutilitytoggle" )->setEnabled( docwin );
	getActionGroup( "picsizetoggle" )->setEnabled( docwin && ! docwin->doc()->calcing() );
	getAction( "save" )->setEnabled( docwin );
	getAction( "saveas" )->setEnabled( docwin );
	getAction( "export" )->setEnabled( docwin && ! docwin->doc()->calcing() );
	getAction( "print" )->setEnabled( docwin && ! docwin->doc()->calcing() );
	getAction( "undo" )->setEnabled( docwin && docwin->doc()->can_undo() && ! docwin->doc()->calcing() );
	getAction( "redo" )->setEnabled( docwin && docwin->doc()->can_redo() && ! docwin->doc()->calcing() );
	getAction( "start" )->setEnabled( docwin && ! docwin->doc()->calcing() );
	getAction( "stop" )->setEnabled( docwin && docwin->doc()->calcing() );
	getAction( "zoomin" )->setEnabled( docwin && ! docwin->doc()->calcing() );
	getAction( "zoomout" )->setEnabled( docwin && ! docwin->doc()->calcing() );
	getAction( "prop" )->setEnabled( docwin && ! docwin->doc()->calcing() );

	getAction( "undo" )->setMenuText( "&Rckgngig: " + ( docwin && docwin->doc()->can_undo() ? docwin->doc()->undo_descr() : "nicht mglich" ) );
	getAction( "redo" )->setMenuText( "&Wiederherstellen: " + ( docwin && docwin->doc()->can_redo() ? docwin->doc()->redo_descr() : "nicht mglich" ) );

	if( docwin )
	{
		switch( docwin->mouseUtility() )
		{
			case DocWindow::zoom:
				getAction( "magnifier" )->setOn( true );
				break;
			case DocWindow::center:
				getAction( "center" )->setOn( true );
				break;
		}

		if( docwin->doc()->picSize() == IntSize( 320, 240 ) )
			getAction( "320" )->setOn( true );
		else if( docwin->doc()->picSize() == IntSize( 640, 480 ) )
			getAction( "640" )->setOn( true );
		else if( docwin->doc()->picSize() == IntSize( 1280, 960 ) )
			getAction( "1280" )->setOn( true );
		else
			getAction( "picsizeuser" )->setOn( true );
	}
}

void ChaosExplorerWindow::windowChanged()
{
	windowActivated(0);
}

void ChaosExplorerWindow::fileNewMandelbrot()
{
	DocWindow* docwin = createDocWindow( new Mandelbrot::Document() );
}

void ChaosExplorerWindow::fileNewFeigenbaum()
{
	DocWindow* docwin = createDocWindow( new Feigenbaum::Document() );
}

void ChaosExplorerWindow::fileOpen()
{
	QString file = QFileDialog::getOpenFileName( QString::null, "ChaosExplorer-Dateien (*.che);;Alle Dateien (*.*)", this, 0, "ChaosExplorer-Datei ffnen" );

	if( file.isEmpty() )
		return;

	if( ! openFile( file ) )
		QMessageBox::critical( this, "ffnen der Datei fehlgeschlagen", "Die Datei\n" + file + "\nkonnte nicht geffnet werden.\nSie ist entweder beschdigt oder sie ist keine ChaosExplorer-Datei." );
}

void ChaosExplorerWindow::fileSave()
{
	DocWindow* docwin = (DocWindow *) m_workspace->activeWindow();

	if( docwin && ! docwin->doc()->saveFile() )
		fileSaveAs();
}

void ChaosExplorerWindow::fileSaveAs()
{
	DocWindow* docwin = (DocWindow *) m_workspace->activeWindow();

	if( ! docwin )
		return;

	QString file = QFileDialog::getSaveFileName( QString::null, "ChaosExplorer-Dateien (*.che);;Alle Dateien (*.*)", this, 0, "ChaosExplorer-Datei speichern" );

	if( ! file.isEmpty() )
	{
		QFileInfo fi( file );
		if( fi.extension().isEmpty() )
		{
			file += ".che";
			fi.setFile( file );
		}
		if( fi.exists() && QMessageBox::warning( this, "Datei berschreiben?", QString( "Mchten Sie die Datei\n%1\nwirklich berschreiben?" ).arg( file ), "&Ja", "&Nein" ) == 1 )
			return;
		if( docwin->doc()->saveFile( file ) )
			setDocWindowCaption( docwin );
		else
			QMessageBox::critical( this, "Speichern der Datei fehlgeschlagen", "Die Datei\n" + file + "\nkonnte nicht gespeichert werden.\nStellen Sie sicher, dass das Verzeichnis nicht schreibgeschtzt ist und genug Speicherplatz vorhanden ist." );
	}
}

void ChaosExplorerWindow::filePrint()
{
	DocWindow* docwin = (DocWindow *) m_workspace->activeWindow();

	if( ! docwin )
		return;
	if( docwin->doc()->pixmap().isNull() )
		return;

	QPrinter printer;
	printer.setFullPage( false );

	QPaintDeviceMetrics printer_metrics( &printer );
	QPaintDeviceMetrics pixmap_metrics( &docwin->doc()->pixmap() );

	if( printer.setup( this ) )
	{
		docwin->doc()->printBitmap( &printer );
	}
}

void ChaosExplorerWindow::fileExport()
{
	DocWindow* docwin = (DocWindow *) m_workspace->activeWindow();

	if( docwin )
	{
		QString file = QFileDialog::getSaveFileName( QString::null, "Bitmap-Dateien (*.bmp);;PNG-Dateien (*.png);;Alle Dateien (*.*)", this, 0, "Bild exportieren" );
		QFileInfo fi( file );
		if( file.isEmpty() )
			return;
		if( fi.extension().isEmpty() )
		{
			file += ".bmp";
			fi.setFile( file );
		}
		if( fi.exists() && QMessageBox::warning( this, "Datei berschreiben?", QString( "Mchten Sie die Datei\n%1\nwirklich berschreiben?" ).arg( file ), "&Ja", "&Nein" ) == 1 )
			return;
		if( ! docwin->doc()->exportBitmap( file ) )
			QMessageBox::critical( this, "Speichern der Datei fehlgeschlagen", "Beim Exportieren des Bildes in die Datei\n" + file + "\nist ein Fehler aufgetreten.\nStellen Sie sicher, dass das Verzeichnis nicht schreibgeschtzt ist und genug Speicherplatz vorhanden ist." );
	}
}

void ChaosExplorerWindow::fileQuit()
{
	qApp->closeAllWindows();
}

void ChaosExplorerWindow::editUndo()
{
	DocWindow* docwin = (DocWindow *) m_workspace->activeWindow();

	if( docwin )
		docwin->doc()->undo();
}

void ChaosExplorerWindow::editRedo()
{
	DocWindow* docwin = (DocWindow *) m_workspace->activeWindow();

	if( docwin )
		docwin->doc()->redo();
}

void ChaosExplorerWindow::editPictureSize( QAction* action )
{
	DocWindow* docwin = (DocWindow *) m_workspace->activeWindow();

	if( docwin )
	{
		IntSize new_size;

		if( action == getAction( "320" ) )
		{
			new_size.width = 320;
			new_size.height = 240;
		}
		else if( action == getAction( "640" ) )
		{
			new_size.width = 640;
			new_size.height = 480;
		}
		else if( action == getAction( "1280" ) )
		{
			new_size.width = 1280;
			new_size.height = 960;
		}
		else // to be sure
		{
			return;
		}

		validatePicSizeRatio( new_size );
		docwin->doc()->setPicSize( new_size );
	}
}

void ChaosExplorerWindow::editPictureSizeUserDef()
{
	DocWindow* docwin = (DocWindow *) m_workspace->activeWindow();

	if( docwin )
	{
		SizeWindow sizewin( this, docwin->doc()->picSize().width, docwin->doc()->picSize().height );
		if( sizewin.exec() == QDialog::Accepted )
		{
			IntSize new_size( sizewin.pictureWidth(), sizewin.pictureHeight() );
			validatePicSizeRatio( new_size );
			docwin->doc()->setPicSize( new_size );
		}
		else
		{
			windowActivated(0); // keep menu checkmark correct
		}
	}
}

void ChaosExplorerWindow::editStartCalc()
{
	DocWindow* docwin = (DocWindow *) m_workspace->activeWindow();

	if( docwin )
		docwin->doc()->startCalc();
}

void ChaosExplorerWindow::editStopCalc()
{
	DocWindow* docwin = (DocWindow *) m_workspace->activeWindow();

	if( docwin )
		docwin->doc()->stopCalc();
}

void ChaosExplorerWindow::editProps()
{
	DocWindow* docwin = (DocWindow *) m_workspace->activeWindow();

	if( docwin )
		docwin->doc()->showProperties( this );
}

void ChaosExplorerWindow::changeUtility( QAction* action )
{
	DocWindow* docwin = (DocWindow *) m_workspace->activeWindow();

	if( ! docwin )
		return;

	DocWindow::MouseUtility newutil = DocWindow::zoom;

	if( action == getAction( "magnifier" ) )
		;
	else if( action == getAction( "center" ) )
		newutil = DocWindow::center;
	else if( action == getAction( "move" ) )
		newutil = DocWindow::move;
	
	docwin->setMouseUtility( newutil );
}

void ChaosExplorerWindow::zoomIn()
{
	// zoom 50% in
	const double zoom = 0.5;

	DocWindow* docwin = (DocWindow *) m_workspace->activeWindow();

	if( ! docwin )
		return;

	DoubleRect coords = docwin->doc()->coords();
	
	double dwidth = coords.width() * zoom / 2;
	double dheight = coords.height() * zoom / 2;

	coords.tl.x += dwidth;
	coords.br.x -= dwidth;
	coords.tl.y += dheight;
	coords.br.y -= dheight;

	docwin->doc()->setCoords( coords );
}

void ChaosExplorerWindow::zoomOut()
{
	// zoom 50% out
	const double zoom = 0.5;

	DocWindow* docwin = (DocWindow *) m_workspace->activeWindow();

	if( ! docwin )
		return;

	DoubleRect coords = docwin->doc()->coords();
	
	double dwidth = coords.width() * zoom / 2;
	double dheight = coords.height() * zoom / 2;

	coords.tl.x -= dwidth;
	coords.br.x += dwidth;
	coords.tl.y -= dheight;
	coords.br.y += dheight;

	docwin->doc()->setCoords( coords );
}

void ChaosExplorerWindow::validatePicSizeRatio( const IntSize& new_size )
{
	DocWindow* docwin = (DocWindow *) m_workspace->activeWindow();

	if( docwin )
	{
		const double old_pic_ratio = (double) docwin->doc()->picSize().width / docwin->doc()->picSize().height;
		const double new_pic_ratio = (double) new_size.width / new_size.height;

		if( new_pic_ratio != old_pic_ratio )
			if( QMessageBox::warning( this, "Falsches Seitenverhltnis", "Die neue Bildgre hat ein anderes Seitenverhltnis als der Bildausschnitt.\nMchten Sie den Bildausschnitt anpassen, um eine verzerrungsfreie Darstellung zu gewhrleisten?", "&Ja", "&Nein" ) == 0 )
			{
				DoubleRect coords = docwin->doc()->coords();
				if( new_pic_ratio >= old_pic_ratio )
				{
					coords.br.x = coords.tl.x + new_pic_ratio * -coords.height();
				}
				else
				{
					coords.br.y = coords.tl.y - 1 / new_pic_ratio * coords.width();
				}
				docwin->doc()->setCoords( coords );
			}
	}
}

bool ChaosExplorerWindow::openFile( QString fileName )
{
	if( ! fileName.isEmpty() )
	{
		// find out what type of document is saved in the file
		QFile file( fileName );
		if( ! file.open( IO_ReadOnly ) ) return false;

		QDataStream ds( &file );

		// read signature to be sure we have a ChaosExplorer file
		char c_str[14];
		ds.readRawBytes( c_str, 14 );
		QString str( c_str );
		if( str != "CHAOSEXPLORER" ) return false;

		// read file format version number (not used yet)
		int ver_maj, ver_min, ver_sub;
		ds >> ver_maj >> ver_min >> ver_sub;

		// read doc type
		int doc_type;
		ds >> doc_type;

		file.close();

		// switch depending on doc type
		DocInterface* doc;
		switch( doc_type )
		{
			case 0: // mandelbrot
				doc = new Mandelbrot::Document();
				break;
			case 1: // feigenbaum
				doc = new Feigenbaum::Document();
				break;
			default:
				return false;
				break;
		}

		if( doc->openFile( fileName ) )
		{
			DocWindow* docwin = createDocWindow( doc );

			if( docwin->width() > m_workspace->width() && docwin->height() > m_workspace->height() )
				docwin->setGeometry( docwin->x(), docwin->y(), 0.9 * m_workspace->width(), 0.9 * m_workspace->height() );

			return true;
		}
		else
		{
			delete doc;
		}
	}
	return false;
}

DocWindow* ChaosExplorerWindow::createDocWindow( DocInterface* newdoc )
{
	DocWindow* docwin = new DocWindow( newdoc, m_workspace );

	setDocWindowCaption( docwin );

	connect( docwin->doc(), SIGNAL( stateChanged() ), SLOT( windowChanged() ) );

	if( m_workspace->windowList().isEmpty() )
		docwin->showMaximized();
	else
		docwin->show();

	return docwin;
}

void ChaosExplorerWindow::setDocWindowCaption( DocWindow* docwin )
{
	if( ! docwin || ! docwin->doc() )
		return;

	QString caption = "";

	if( docwin->doc()->isA( "Mandelbrot::Document" ) )
	{
		static int doc_counter = 0;
		caption = "Mandelbrot - %1";
		if( docwin->doc()->fileName().isEmpty() )
			caption = caption.arg( ++doc_counter );
		else
			caption = caption.arg( QFileInfo( docwin->doc()->fileName() ).fileName() );
	}
	else if( docwin->doc()->isA( "Feigenbaum::Document" ) )
	{
		static int doc_counter = 0;
		caption = "Feigenbaum - %1";
		if( docwin->doc()->fileName().isEmpty() )
			caption = caption.arg( ++doc_counter );
		else
			caption = caption.arg( QFileInfo( docwin->doc()->fileName() ).fileName() );
	}

	docwin->setCaption( caption );
}

void ChaosExplorerWindow::updateWindowMenu()
{
	QPopupMenu* menu = (QPopupMenu*) child( "windowmenu" );
	if( ! menu )
		return;

	QWidgetList list = m_workspace->windowList();

	menu->clear();

	menu->setItemEnabled( menu->insertItem( QPixmap( cascade_xpm ), "ber&lappend", m_workspace, SLOT( cascade() ) ), list.count() > 0 );
	menu->setItemEnabled( menu->insertItem( QPixmap( tile_xpm ), "&Nebeneinander", m_workspace, SLOT( tile() ) ), list.count() > 0 );

	menu->insertSeparator();

	if( list.count() > 0 )
	{
		int i = 1;
		for( QWidget* window = list.first(); window != 0; i++, window = list.next() )
			menu->insertItem( QString( "%1 %2" ).arg(i).arg( window->caption() ), window, SLOT( setFocus() ) );
	}
	else
	{
		menu->setItemEnabled( menu->insertItem( "<Keine Fenster vorhanden>" ), false );
	}
}

void ChaosExplorerWindow::showInfo()
{
	InfoWindow infodlg( this );
	infodlg.exec();
}
