/*
 * ChaosExplorer source file
 * Copyright (C) 2002-2003 by Roland Riegel <feedback@roland-riegel.de>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * $Id$
 *
 */

#include "colorfadewidget.h"

#include <qpopupmenu.h>
#include <qcolordialog.h>
#include <qpainter.h>
#include <qaction.h>

ColorFadeWidget::ColorFadeWidget( QWidget* parent, const char* name )
	: QVBox( parent, name )
{

	setFrameStyle( QFrame::Sunken | QFrame::Panel );
	setMargin(5);

	m_fade = new UserWidget( this, 0, Qt::WResizeNoErase );
	connect( m_fade, SIGNAL( paint( QPaintEvent* ) ), SLOT( fadePaint( QPaintEvent* ) ) );
	connect( m_fade, SIGNAL( resized( QResizeEvent* ) ), SLOT( fadeResized( QResizeEvent* ) ) );
	connect( m_fade, SIGNAL( mouseMoved( QMouseEvent* ) ), SLOT( arrowsMouseMoved( QMouseEvent* ) ) );
	connect( m_fade, SIGNAL( mousePressed( QMouseEvent* ) ), SLOT( arrowsMousePressed( QMouseEvent* ) ) );
	connect( m_fade, SIGNAL( mouseReleased( QMouseEvent* ) ), SLOT( arrowsMouseReleased( QMouseEvent* ) ) );
	connect( m_fade, SIGNAL( mouseDoubleClicked( QMouseEvent* ) ), SLOT( arrowsMouseDoubleClicked( QMouseEvent* ) ) );

	m_arrows = new UserWidget( this );
	connect( m_arrows, SIGNAL( paint( QPaintEvent* ) ), SLOT( arrowsPaint( QPaintEvent* ) ) );
	connect( m_arrows, SIGNAL( mouseMoved( QMouseEvent* ) ), SLOT( arrowsMouseMoved( QMouseEvent* ) ) );
	connect( m_arrows, SIGNAL( mousePressed( QMouseEvent* ) ), SLOT( arrowsMousePressed( QMouseEvent* ) ) );
	connect( m_arrows, SIGNAL( mouseReleased( QMouseEvent* ) ), SLOT( arrowsMouseReleased( QMouseEvent* ) ) );
	connect( m_arrows, SIGNAL( mouseDoubleClicked( QMouseEvent* ) ), SLOT( arrowsMouseDoubleClicked( QMouseEvent* ) ) );
	m_arrows->setBackgroundColor( m_arrows->colorGroup().dark() );
	m_arrows->setFixedHeight(8);

	m_curr_arrow = -1;

	//create popup menu

	m_popup = new QPopupMenu( this );

	QAction* action = 0;
	action = new QAction( "Neu", "Neu", 0, this );
	action->addTo( m_popup );

	action = new QAction( "Loeschen", "Lschen", 0, this );
	action->addTo( m_popup );

	m_popup->insertSeparator();

	action = new QAction( "Bearbeiten", "Bearbeiten...", 0, this );
	action->addTo( m_popup );


}

ColorFadeWidget::~ColorFadeWidget()
{
}

ColorFade ColorFadeWidget::colorFade() const
{
	return m_colorfade;
}

void ColorFadeWidget::setColorFade( const ColorFade& colorfade )
{
	m_colorfade = colorfade;
}

void ColorFadeWidget::drawFade()
{

	m_pixmap.resize( m_fade->size() );

	QPainter painter( &m_pixmap );

	int red, green, blue;
	for( int x = 0; x < m_fade->width(); x++ )
	{
		double pos = (double) x / ( m_fade->width() - 1 );

		m_colorfade.colorbypos( pos, red, green, blue );

		QColor color( red, green, blue );

		painter.setPen( color );
		painter.drawLine( x, 0, x, m_fade->height() );
	}

}

void ColorFadeWidget::drawArrows()
{
	QPainter painter( m_arrows );
	painter.setPen( Qt::black );
	painter.setBrush( Qt::black );

	painter.fillRect( 0, 0, m_arrows->width(), m_arrows->height(), m_arrows->backgroundColor() );

	for( ColorFade::size_type i = 0; i < m_colorfade.size(); i++ )
	{
		QPointArray array;
		int x1 = m_colorfade[i].pos * ( m_arrows->width() - 1 );
		int y1 = 0;
		int x2 = x1 + m_arrows->height() / 2;
		int y2 = m_arrows->height() - 1;
		int x3 = x1 - m_arrows->height() / 2;
		int y3 = y2;
		array.setPoints( 3, x1, y1, x2, y2, x3, y3 );

		painter.drawPolygon( array );
	}
}

void ColorFadeWidget::fadePaint( QPaintEvent* e )
{
	bitBlt( m_fade, 0, 0, &m_pixmap, 0, 0, m_fade->width(), m_fade->height(), Qt::CopyROP );
}

void ColorFadeWidget::fadeResized( QResizeEvent* e )
{
	drawFade();
}

void ColorFadeWidget::arrowsPaint( QPaintEvent* e )
{
	drawArrows();
}

void ColorFadeWidget::arrowsMouseMoved( QMouseEvent* e )
{
	if( m_curr_arrow != -1 )
	{
		double new_pos = (double) e->x() / ( m_arrows->width() - 1 );

		if( new_pos < 0 ) new_pos = 0;
		if( new_pos > 1 ) new_pos = 1;

		if( m_curr_arrow == 0 )
		{
			new_pos = new_pos > m_colorfade[ m_curr_arrow + 1 ].pos ? m_colorfade[ m_curr_arrow + 1 ].pos : new_pos;
		}
		else if( m_curr_arrow == m_colorfade.size() - 1 )
		{
			new_pos = new_pos < m_colorfade[ m_curr_arrow - 1 ].pos ? m_colorfade[ m_curr_arrow - 1 ].pos : new_pos;
		}
		else
		{
			new_pos = new_pos > m_colorfade[ m_curr_arrow + 1 ].pos ? m_colorfade[ m_curr_arrow + 1 ].pos : new_pos;
			new_pos = new_pos < m_colorfade[ m_curr_arrow - 1 ].pos ? m_colorfade[ m_curr_arrow - 1 ].pos : new_pos;
		}

		m_colorfade[ m_curr_arrow ].pos = new_pos;

		drawArrows();
		drawFade();
		fadePaint(0);
	}
}

void ColorFadeWidget::arrowsMousePressed( QMouseEvent* e )
{
	if( e->button() == Qt::LeftButton )
		m_curr_arrow = arrowByPos( (double) e->x() / ( m_arrows->width() - 1 ) );
}

void ColorFadeWidget::arrowsMouseReleased( QMouseEvent* e )
{
	switch( e->button() )
	{
		case Qt::LeftButton:
			m_curr_arrow = -1;
			break;
		case Qt::RightButton:
			double mouse_pos = (double) e->x() / ( m_arrows->width() - 1 );
			switch( m_popup->indexOf( m_popup->exec( e->globalPos() ) ) )
			{
				case 0:
					addArrow( mouse_pos );
					break;
				case 1:
					removeArrow( mouse_pos );
					break;
				case 3:
					int arrow = arrowByPos( mouse_pos );
					QColor old_color( m_colorfade[ arrow ].red, m_colorfade[ arrow ].green, m_colorfade[ arrow ].blue );
					QColor color = QColorDialog::getColor( old_color, this );
					if( color.isValid() )
					{
						m_colorfade[ arrow ].red = color.red();
						m_colorfade[ arrow ].green = color.green();
						m_colorfade[ arrow ].blue = color.blue();
					}
					break;
			}

			drawArrows();
			drawFade();
			fadePaint(0);
			break;
	}
}

void ColorFadeWidget::arrowsMouseDoubleClicked( QMouseEvent* e )
{
	if( e->button() == Qt::LeftButton )
	{
		double mouse_pos = (double) e->x() / ( m_arrows->width() - 1 );
		addArrow( mouse_pos );

		drawArrows();
		drawFade();
		fadePaint(0);
	}
}

void ColorFadeWidget::addArrow( double pos )
{
	int red, green, blue;
	m_colorfade.colorbypos( pos, red, green, blue );
	m_colorfade.add( ColorStep( red, green, blue, pos ) );
}

void ColorFadeWidget::removeArrow( double pos )
{
	m_colorfade.remove( arrowByPos( pos ) );
}

int ColorFadeWidget::arrowByPos( double pos )
{
	if( pos <= m_colorfade[0].pos )
		return 0;
	else if( pos >= m_colorfade[ m_colorfade.size() - 1 ].pos )
		return m_colorfade.size() - 1;
	else
		for( ColorFade::size_type i = 0; i < m_colorfade.size() - 1; i++ )
		{
			if( pos >= m_colorfade[i].pos && pos <= m_colorfade[i].pos + ( m_colorfade[ i + 1 ].pos - m_colorfade[i].pos ) / 2 )
			{
				return i;
			}
			if( pos >= m_colorfade[i].pos + ( m_colorfade[ i + 1 ].pos - m_colorfade[i].pos ) / 2 && pos <= m_colorfade[ i + 1 ].pos )
			{
				return i + 1;
			}
		}

	return 0;
}