/*
 * ChaosExplorer source file
 * Copyright (C) 2002-2003 by Roland Riegel <feedback@roland-riegel.de>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * $Id$
 *
 */

#if !defined(COLORFADEWIDGET_H)
#define COLORFADEWIDGET_H

#include "vartypes.h"

#include <qwidget.h>
#include <qvbox.h>
#include <qpixmap.h>

class QPopupMenu;

class UserWidget : public QWidget
{
	Q_OBJECT
public:
	UserWidget( QWidget* parent = 0, const char* name = 0, WFlags f = 0 ) : QWidget( parent, name, f ) {}
	~UserWidget() {}

signals:
	void mousePressed( QMouseEvent* e );
	void mouseReleased( QMouseEvent* e );
	void mouseDoubleClicked( QMouseEvent* e );
	void mouseMoved( QMouseEvent* e );
	void resized( QResizeEvent* e );
	void paint( QPaintEvent* e );

private:
	void mousePressEvent( QMouseEvent* e ) { emit( mousePressed(e) ); }
	void mouseReleaseEvent( QMouseEvent* e ) { emit( mouseReleased(e) ); }
	void mouseDoubleClickEvent( QMouseEvent* e ) { emit( mouseDoubleClicked(e) ); }
	void mouseMoveEvent( QMouseEvent* e ) { emit( mouseMoved(e) ); }
	void resizeEvent( QResizeEvent* e ) { emit( resized(e) ); }
	void paintEvent( QPaintEvent* e ) { emit( paint(e) ); }

};

class ColorFadeWidget : public QVBox
{
	Q_OBJECT
public:
	ColorFadeWidget( QWidget* parent = 0, const char* name = 0 );
	~ColorFadeWidget();

	ColorFade colorFade() const;
	void setColorFade( const ColorFade& colorfade );

private slots:
	void fadePaint( QPaintEvent* e );
	void fadeResized( QResizeEvent* e );
	void arrowsPaint( QPaintEvent* e );
	void arrowsMouseMoved( QMouseEvent* e );
	void arrowsMousePressed( QMouseEvent* e );
	void arrowsMouseReleased( QMouseEvent* e );
	void arrowsMouseDoubleClicked( QMouseEvent* e );

private:
	void drawFade();
	void drawArrows();
	int arrowByPos( double pos );
	void addArrow( double pos );
	void removeArrow( double pos );

	UserWidget* m_fade;
	UserWidget* m_arrows;

	ColorFade m_colorfade;
	QPixmap m_pixmap;

	ColorFade::size_type m_curr_arrow;

	QPopupMenu* m_popup;

};

#endif // !defined(COLORFADEWIDGET_H)
