/*
 * ChaosExplorer source file
 * Copyright (C) 2002-2003 by Roland Riegel <feedback@roland-riegel.de>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * $Id$
 *
 */

#if !defined(DOCINTERFACE_H)
#define DOCINTERFACE_H

#include <qobject.h>

class QPrinter;
class QPixmap;
class QWidget;
class QString;
template<class T> class Size;
typedef Size<int> IntSize;
template<class T> class Rect;
typedef Rect<double> DoubleRect;

class DocInterface : public QObject
{
	Q_OBJECT
public:
	virtual bool openFile( QString fileName ) = 0;
	virtual bool saveFile( QString fileName = "" ) = 0;
	virtual bool exportBitmap( QString fileName ) = 0;

	virtual QString fileName() const = 0;

	virtual void printBitmap( const QPrinter* printer ) = 0;

	virtual void setPicSize( const IntSize& size ) = 0;
	virtual const IntSize& picSize() const = 0;

	virtual void setCoords( const DoubleRect& coords ) = 0;
	virtual const DoubleRect& coords() const = 0;

	virtual void startCalc() = 0;
	virtual void stopCalc() = 0;
	virtual bool calcing() const = 0;

	virtual void undo() = 0;
	virtual void redo() = 0;
	virtual bool can_undo() const = 0;
	virtual bool can_redo() const = 0;
	virtual QString undo_descr() const = 0;
	virtual QString redo_descr() const = 0;

	virtual QPixmap& pixmap() = 0;

	virtual void showProperties( QWidget* parent = 0 ) = 0;

signals:
	void stateChanged();

	void resized();

	void rectPainted( const QRect& rect );

	void startedCalc();
	void stoppedCalc();

};

#endif // !defined(DOCINTERFACE_H)
