/*
 * ChaosExplorer source file
 * Copyright (C) 2002-2003 by Roland Riegel <feedback@roland-riegel.de>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * $Id$
 *
 */

#if !defined(DOCWINDOW_H)
#define DOCWINDOW_H

#include "vartypes.h"

#include <qmainwindow.h>
#include <qdatetime.h>

class QLabel;
class QScrollView;
class DocInterface;
class DrawWidget;

class DocWindow : public QMainWindow
{
	Q_OBJECT
public:

	enum MouseUtility
	{
		zoom,
		center,
		move
	};

	DocWindow( DocInterface* doc, QWidget* parent = 0 );
	~DocWindow();

	DocInterface* doc();

	MouseUtility mouseUtility();
	void setMouseUtility( MouseUtility new_mouseutility );

private slots:
	void docResized();
	void refresh( const QRect& rect );
	void mouseDownLeft( const QPoint& point );
	void mouseUpLeft( const QPoint& point );
	void mouseDownRight( const QPoint& point );
	void mouseMove( const QPoint& point );
	void mouseLeave();
	void startClock();
	void stopClock();

private:
	DoublePoint translatePoint( const IntPoint& point );
	void drawZoomRect();

	QScrollView* m_scrollview;
	DrawWidget* m_docwidget;

	QLabel* m_status_coords;
	QLabel* m_status_msg;

	QTime m_time;

	DocInterface* m_doc;

	MouseUtility m_mouseutility;

	bool m_zooming;
	bool m_moving;
	IntRect m_rect_mouse;

};

#endif // !defined(DOCWINDOW_H)
