/*
 * ChaosExplorer source file
 * Copyright (C) 2002-2003 by Roland Riegel <feedback@roland-riegel.de>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * $Id$
 *
 */

#ifndef FEIGENBAUM_ACTION_H
#define FEIGENBAUM_ACTION_H

#include <qstring.h>

#include "feigenbaum_calcinputdata.h"
#include "vartypes.h"

namespace Feigenbaum
{

	class Action
	{
	public:
		Action() {}
		virtual ~Action() {}

		virtual QString description() const = 0;
		virtual void undo( Handle<CalcInputData> input ) const = 0;
		virtual void redo( Handle<CalcInputData> input ) const = 0;
	};

	template<class T>
	class ActionTempl : public Action
	{
	public:
		ActionTempl( const T& new_val, const T& old_val )
			: Action(), m_new( new_val ), m_old( old_val )
		{
		}

	protected:
		T m_new;
		T m_old;
	};

	class CoordsAction : public ActionTempl<DoubleRect>
	{
	public:
		CoordsAction( const DoubleRect& new_val, const DoubleRect& old_val )
			: ActionTempl<DoubleRect>( new_val, old_val )
		{
		}

		QString description() const
		{
			return "Bildausschnitt";
		}

		void undo( Handle<CalcInputData> input ) const
		{
			input->setCoords( m_old );
		}

		void redo( Handle<CalcInputData> input ) const
		{
			input->setCoords( m_new );
		}
	};

	class PicSizeAction : public ActionTempl<IntSize>
	{
	public:
		PicSizeAction( const IntSize& new_val, const IntSize& old_val )
			: ActionTempl<IntSize>( new_val, old_val )
		{
		}

		QString description() const
		{
			return "Bildgre";
		}

		void undo( Handle<CalcInputData> input ) const
		{
			input->setPicSize( m_old );
		}

		void redo( Handle<CalcInputData> input ) const
		{
			input->setPicSize( m_new );
		}
	};

	class MaxIterationsAction : public ActionTempl<unsigned int>
	{
	public:
		MaxIterationsAction( const unsigned int& new_val, const unsigned int& old_val )
			: ActionTempl<unsigned int>( new_val, old_val )
		{
		}

		QString description() const
		{
			return "Max. Iterationen";
		}

		void undo( Handle<CalcInputData> input ) const
		{
			input->setMaxIterations( m_old );
		}

		void redo( Handle<CalcInputData> input ) const
		{
			input->setMaxIterations( m_new );
		}
	};

	class PaintIterationsAction : public ActionTempl<unsigned int>
	{
	public:
		PaintIterationsAction( const unsigned int& new_val, const unsigned int& old_val )
			: ActionTempl<unsigned int>( new_val, old_val )
		{
		}

		QString description() const
		{
			return "Dargest. Iterationen";
		}

		void undo( Handle<CalcInputData> input ) const
		{
			input->setPaintIterations( m_old );
		}

		void redo( Handle<CalcInputData> input ) const
		{
			input->setPaintIterations( m_new );
		}
	};

	class StartValueAction : public ActionTempl<double>
	{
	public:
		StartValueAction( const double& new_val, const double& old_val )
			: ActionTempl<double>( new_val, old_val )
		{
		}

		QString description() const
		{
			return "Startwert fr Iteration";
		}

		void undo( Handle<CalcInputData> input ) const
		{
			input->setStartValue( m_old );
		}

		void redo( Handle<CalcInputData> input ) const
		{
			input->setStartValue( m_new );
		}
	};

}

#endif

