/*
 * ChaosExplorer source file
 * Copyright (C) 2002-2003 by Roland Riegel <feedback@roland-riegel.de>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * $Id$
 *
 */

#include "feigenbaum_calc.h"

Feigenbaum::Calc::Calc( Handle<CalcInputData> input, Handle<CalcResult> output )
	: m_calcing( false ), m_input( input ), m_output( output )
{
	connect( m_input.get(), SIGNAL( picSizeChanged() ), m_output.get(), SLOT( deleteData() ) );
	connect( m_input.get(), SIGNAL( maxIterationsChanged() ), m_output.get(), SLOT( deleteData() ) );
	connect( m_input.get(), SIGNAL( paintIterationsChanged() ), m_output.get(), SLOT( deleteData() ) );
	connect( m_input.get(), SIGNAL( startValueChanged() ), m_output.get(), SLOT( deleteData() ) );
	connect( m_input.get(), SIGNAL( coordsChanged() ), m_output.get(), SLOT( deleteData() ) );
}

Feigenbaum::Calc::~Calc()
{
}

void Feigenbaum::Calc::run()
{
	m_calcing = true;
	
	emit started();

	m_output->pixmap().resize( m_input->picSize().width, m_input->picSize().height );
	m_output->deleteData();

	for( int x = 0; x < m_input->picSize().width && m_calcing; x++ )
	{
		const double u = m_input->coords().tl.x + (double) m_input->coords().width() / m_input->picSize().width * x;
		double result = m_input->startValue();	
		for( int iter = 0; iter < m_input->maxIterations(); iter++ )
		{
			result = u * result * ( 1 - result );
			//result = result * result + u;
			if( iter >= m_input->maxIterations() - m_input->paintIterations() )
			{
				const double percent = ( result - m_input->coords().br.y ) / -m_input->coords().height();
				const int y = ( 1 - percent ) * m_input->picSize().height;

				m_output->setPixel( IntPoint( x, y ) );
			}
		}

		emit updated( QRect( x, 0, 1, m_input->picSize().height ) );
	}

	if( m_calcing )
	{
		m_calcing = false;
		emit finished();
	}
	else
	{
		emit stopped();
	}
}

void Feigenbaum::Calc::stop()
{
	m_calcing = false;
}

bool Feigenbaum::Calc::calcing()
{
	return m_calcing;
}

