/*
 * ChaosExplorer source file
 * Copyright (C) 2002-2003 by Roland Riegel <feedback@roland-riegel.de>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * $Id$
 *
 */

#ifndef FEIGENBAUM_CALCINPUTDATA_H
#define FEIGENBAUM_CALCINPUTDATA_H

#include <qobject.h>
#include "vartypes.h"

namespace Feigenbaum
{
	class CalcInputData : public QObject
	{
		Q_OBJECT
	public:
		CalcInputData();

		void setPicSize( const IntSize& new_picsize );
		const IntSize& picSize() const;

		void setCoords( const DoubleRect& new_coords );
		const DoubleRect& coords() const;

		void setMaxIterations( unsigned int new_maxiterations );
		unsigned int maxIterations() const;

		void setPaintIterations( unsigned int new_paintiterations );
		unsigned int paintIterations() const;

		void setStartValue( double new_startvalue );
		double startValue() const;

	signals:
		void picSizeChanged();
		void coordsChanged();
		void maxIterationsChanged();
		void paintIterationsChanged();
		void startValueChanged();
		
	private:
		IntSize m_picSize;
		DoubleRect m_coords;
		unsigned int m_maxIterations;
		unsigned int m_paintIterations;
		double m_startValue;
	};
}

inline Feigenbaum::CalcInputData::CalcInputData()
	: QObject(), m_picSize( 320, 240 ),
	  m_coords(  1, 2, 4, -1  ), m_maxIterations( 1000 ),
	  m_paintIterations( 100 ), m_startValue( 0.1 )
{
}

inline void Feigenbaum::CalcInputData::setPicSize( const IntSize& new_picsize )
{
	if( new_picsize == m_picSize ) return;

	m_picSize = new_picsize;
	emit picSizeChanged();
}

inline const IntSize& Feigenbaum::CalcInputData::picSize() const
{
	return m_picSize;
}

inline void Feigenbaum::CalcInputData::setCoords( const DoubleRect& new_coords )
{
	if( new_coords.width() == 0 || new_coords.height() == 0 ) return;

	m_coords = new_coords;
	emit coordsChanged();
}

inline const DoubleRect& Feigenbaum::CalcInputData::coords() const
{
	return m_coords;
}

inline void Feigenbaum::CalcInputData::setMaxIterations( unsigned int new_maxiterations )
{
	if( new_maxiterations == m_maxIterations ) return;

	m_maxIterations = new_maxiterations;
	emit maxIterationsChanged();
}

inline unsigned int Feigenbaum::CalcInputData::maxIterations() const
{
	return m_maxIterations;
}

inline void Feigenbaum::CalcInputData::setPaintIterations( unsigned int new_paintiterations )
{
	if( new_paintiterations == m_paintIterations ) return;

	m_paintIterations = new_paintiterations;
	emit paintIterationsChanged();
}

inline unsigned int Feigenbaum::CalcInputData::paintIterations() const
{
	return m_paintIterations;
}

inline void Feigenbaum::CalcInputData::setStartValue( double new_startvalue )
{
	if( new_startvalue == m_startValue ) return;

	m_startValue = new_startvalue;
	emit startValueChanged();
}

inline double Feigenbaum::CalcInputData::startValue() const
{
	return m_startValue;
}

#endif
