/*
 * ChaosExplorer source file
 * Copyright (C) 2002-2003 by Roland Riegel <feedback@roland-riegel.de>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * $Id$
 *
 */

#ifndef MANDELBROT_ACTIONCOLLECTION_H
#define MANDELBROT_ACTIONCOLLECTION_H

#include <stack>
using std::stack;

#include "mandelbrot_action.h"
#include "vartypes.h"

namespace Mandelbrot
{
	class ActionCollection
	{
	public:
		ActionCollection( Handle<CalcInputData> input );
		~ActionCollection();

		void add( Action* new_action );

		void undo();
		void redo();

		bool can_undo() const;
		bool can_redo() const;

		QString undo_description() const;
		QString redo_description() const;

	private:
		void clearStack( stack<Action*>& stack_to_clear );

		stack<Action*> m_undo;
		stack<Action*> m_redo;

		Handle<CalcInputData> m_input;
	};

	inline ActionCollection::ActionCollection( Handle<CalcInputData> input )
	{
		m_input = input;
	}

	inline ActionCollection::~ActionCollection()
	{
		clearStack( m_undo );
		clearStack( m_redo );
	}

	inline void ActionCollection::add( Action* new_action )
	{
		m_undo.push( new_action );
		clearStack( m_redo );
	}

	inline void ActionCollection::undo()
	{
		if( can_undo() )
		{
			m_redo.push( m_undo.top() );
			m_undo.pop();
			m_redo.top()->undo( m_input );
		}
	}

	inline void ActionCollection::redo()
	{
		if( can_redo() )
		{
			m_undo.push( m_redo.top() );
			m_redo.pop();
			m_undo.top()->redo( m_input );
		}
	}

	inline bool ActionCollection::can_undo() const
	{
		return ! m_undo.empty();
	}

	inline bool ActionCollection::can_redo() const
	{
		return ! m_redo.empty();
	}

	inline QString ActionCollection::undo_description() const
	{
		return can_undo() ? m_undo.top()->description() : "";
	}

	inline QString ActionCollection::redo_description() const
	{
		return can_redo() ? m_redo.top()->description() : "";
	}

	inline void ActionCollection::clearStack( stack<Action*>& stack_to_clear )
	{
		while( ! stack_to_clear.empty() )
		{
			delete stack_to_clear.top();
			stack_to_clear.pop();
		}
	}
}

#endif

