/*
 * ChaosExplorer source file
 * Copyright (C) 2002-2003 by Roland Riegel <feedback@roland-riegel.de>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * $Id$
 *
 */

#include "mandelbrot_calcresult.h"

IntMatrix& Mandelbrot::CalcResult::resultMatrix()
{
	return m_resultMatrix;
}

vector<QColor>& Mandelbrot::CalcResult::palette()
{
	return m_palette;
}

QPixmap& Mandelbrot::CalcResult::pixmap()
{
	return m_pixmap;
}

bool Mandelbrot::CalcResult::exportBitmap( QString fileName )
{
	QFileInfo fi( fileName );
	if( fi.extension().lower() == "png" )
		return m_pixmap.save( fileName, "PNG" );
	else
		return m_pixmap.save( fileName, "BMP" );
}

void Mandelbrot::CalcResult::printPixmap( const QPrinter* printer )
{
	QPaintDeviceMetrics printer_metrics( printer );
	QPaintDeviceMetrics pixmap_metrics( &m_pixmap );

	double new_printer_width = (double) pixmap_metrics.width() / pixmap_metrics.widthMM() * printer_metrics.widthMM();
	double new_printer_height = (double) pixmap_metrics.height() / pixmap_metrics.heightMM() * printer_metrics.heightMM();

	if( pixmap_metrics.width() > new_printer_width )
	{
		new_printer_height *= (double) pixmap_metrics.width() / new_printer_width;
		new_printer_width = pixmap_metrics.width();
	}
	if( pixmap_metrics.height() > new_printer_height )
	{
		new_printer_width *= (double) pixmap_metrics.height() / new_printer_height;
		new_printer_height = pixmap_metrics.height();
	}

	QPainter painter( printer );
	painter.setWindow( 0, 0, new_printer_width, new_printer_height );
	painter.drawPixmap( 0, 0, m_pixmap );
}

void Mandelbrot::CalcResult::resize( const IntSize& size )
{
	m_resultMatrix.resize( (unsigned int) size.width, (unsigned int) size.height, -1 );
	m_pixmap.resize( size.width, size.height );
	m_pixmap.fill( Qt::black );
	emit pixmapChanged( m_pixmap.rect() );
}

void Mandelbrot::CalcResult::rectPainted( const IntRect& rect )
{
	if( m_resultMatrix.width() != m_pixmap.width() || m_resultMatrix.height() != m_pixmap.height() )
	{
		m_pixmap.resize( m_resultMatrix.width(), m_resultMatrix.height() );
		m_pixmap.fill( Qt::black );
	}
	
	if( rect.br.x < m_resultMatrix.width() && rect.br.y < m_resultMatrix.height() )
	{
		QPainter painter( &m_pixmap );
		for( int y = rect.tl.y; y <= rect.br.y; y++ )
		{
			for( int x = rect.tl.x; x <= rect.br.x; x++ )
			{
				int& n = m_resultMatrix( x, y );
				if( n != -1 )
				{
					painter.setPen( m_palette[ n % m_palette.size() ] );
					painter.drawLine( x, y, x, y ); // painter.drawPoint( x, y );
				} 
			}
		}
	}

	emit pixmapChanged( QRect( rect.tl.x, rect.tl.y, rect.width() + 1, rect.height() + 1 ) );
}

void Mandelbrot::CalcResult::deleteData()
{
	m_resultMatrix.resize(0);
	m_palette.resize(0);
	m_pixmap.resize( 0, 0 );

	emit pixmapChanged( QRect( 0, 0, 0, 0 ) );
}

