/*
 * ChaosExplorer source file
 * Copyright (C) 2002-2003 by Roland Riegel <feedback@roland-riegel.de>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * $Id$
 *
 */

#if !defined(MANDELBROT_PROPDIALOG_H)
#define MANDELBROT_PROPDIALOG_H

#include <qdialog.h>
#include <qtabwidget.h>
#include <qvbox.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qgroupbox.h>
#include <qvgroupbox.h>
#include <qlayout.h>
#include <qvalidator.h>
#include <qtextstream.h>

#include "colorfadewidget.h"
#include "mandelbrot_calcinputdata.h"
#include "vartypes.h"

namespace Mandelbrot
{
	class PropDialog : public QDialog
	{
		Q_OBJECT
	public:
		PropDialog( QWidget* parent = 0, const char* name = 0 );
		~PropDialog();

		void setMaxIterations( int iterations );
		int maxIterations() const;

		void setExponent( double exponent );
		double exponent() const;

		void setOptimization( CalcInputData::Optimization opt );
		CalcInputData::Optimization optimization() const;

		void setCoords( const DoubleRect& coords );
		DoubleRect coords() const;

		void setColorFade( const ColorFade& colorfade );
		ColorFade colorFade() const;

	private slots:
		void optActivated( int index );

	private:
		QSpinBox* m_max_iterations;
		QLineEdit* m_exponent;
		QComboBox* m_opt;
		QLabel* m_opt_descr;
		QLineEdit* m_x1;
		QLineEdit* m_y1;
		QLineEdit* m_x2;
		QLineEdit* m_y2;
		ColorFadeWidget* m_fade;
		QSpinBox* m_repeat_interval;

	};
}

#endif // !defined(MANDELBROT_PROPDIALOG_H)
